"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Path = require("path");
const FS = require("fs");
const globby = require("globby");
const core_1 = require("@angular-devkit/core");
const normalize_asset_patterns_1 = require("@angular-devkit/build-angular/src/utils/normalize-asset-patterns");
const log = require("ng-packagr/lib/util/log");
const build_1 = require("../build");
function copyFilesTask(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const globalContext = context.context();
        if (context.epNode.data.entryPoint.isSecondaryEntryPoint) {
            return;
        }
        const { builderContext, builderConfig } = globalContext;
        const root = builderContext.workspace.root;
        const projectRoot = core_1.resolve(root, builderConfig.root);
        const host = new core_1.virtualFs.AliasHost(builderContext.host);
        const syncHost = new core_1.virtualFs.SyncDelegateHost(host);
        const assets = normalize_asset_patterns_1.normalizeAssetPatterns(builderConfig.options.tasks.data.copyFile.assets, syncHost, root, projectRoot, builderConfig.sourceRoot);
        const copyPatterns = buildCopyPatterns(root, assets);
        const copyOptions = { ignore: ['.gitkeep', '**/.DS_Store', '**/Thumbs.db'] };
        log.info('Copying assets');
        const w = copyPatterns.map(pattern => {
            const fullPattern = pattern.context + pattern.from.glob;
            const opts = Object.assign({}, copyOptions, { dot: pattern.from.dot });
            return globby(fullPattern, opts).then(entries => {
                entries.forEach(entry => {
                    const cleanFilePath = entry.replace(pattern.context, '');
                    const to = Path.resolve(root, pattern.to, cleanFilePath);
                    const pathToFolder = Path.dirname(to);
                    pathToFolder.split('/').reduce((p, folder) => {
                        p += folder + '/';
                        if (!FS.existsSync(p)) {
                            FS.mkdirSync(p);
                        }
                        return p;
                    }, '');
                    FS.copyFileSync(entry, to);
                    log.success(` - from: ${entry}`);
                    log.success(` - to: ${to}`);
                });
            });
        });
        try {
            yield Promise.all(w);
        }
        catch (err) {
            builderContext.logger.error(err.toString());
            throw err;
        }
    });
}
function buildCopyPatterns(root, assets) {
    return assets.map(asset => {
        // Resolve input paths relative to workspace root and add slash at the end.
        asset.input = Path.resolve(root, asset.input).replace(/\\/g, '/');
        asset.input = asset.input.endsWith('/') ? asset.input : asset.input + '/';
        asset.output = asset.output.endsWith('/') ? asset.output : asset.output + '/';
        if (asset.output.startsWith('..')) {
            const message = 'An asset cannot be written to a location outside of the output path.';
            throw new Error(message);
        }
        return {
            context: asset.input,
            // Now we remove starting slash to make Webpack place it from the output root.
            to: asset.output.replace(/^\//, ''),
            ignore: asset.ignore,
            from: {
                glob: asset.glob,
                dot: true,
            },
        };
    });
}
let CopyFile = class CopyFile {
};
CopyFile = __decorate([
    build_1.Job({
        schema: Path.resolve(__dirname, 'copy-file.json'),
        selector: 'copyFile',
        hooks: {
            writePackage: {
                before: copyFilesTask
            }
        }
    })
], CopyFile);
exports.CopyFile = CopyFile;
//# sourceMappingURL=copy-file.js.map