"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Path = require("path");
const ts = require("typescript");
const ng = require("@angular/compiler-cli");
const tsconfig_1 = require("ng-packagr/lib/ts/tsconfig");
const nodes_1 = require("ng-packagr/lib/ng-v5/nodes");
const write_package_transform_1 = require("ng-packagr/lib/ng-v5/entry-point/write-package.transform");
const path_1 = require("ng-packagr/lib/util/path");
const log = require("ng-packagr/lib/util/log");
const build_1 = require("../build");
function readTsConfig(configFile) {
    const rawConfigRead = ts.readConfigFile(configFile, ts.sys.readFile);
    if (rawConfigRead.error) {
        return {
            options: {},
            fileNames: [],
            errors: [rawConfigRead.error],
        };
    }
    return ts.parseJsonConfigFileContent(rawConfigRead.config, ts.sys, Path.dirname(configFile), undefined, Path.basename(configFile));
}
/**
 * Run after the original initTsConfig and load the user tsconfig override, if given or the root tsConfig file.
 * We reload the configuration from scratch and create configuration parameters for a simple tsc compilation without the angular compiler stuff.
 * We run "after" and not "replace" because there are a lot of areas depending on the angular compiler `ParsedConfiguration` object.
 */
function initTsConfig(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const globalContext = context.context();
        const { builderConfig } = globalContext;
        const nodeLib = builderConfig.options.tasks.data.nodeLib || {};
        const tsConfigPath = (nodeLib && nodeLib.tsConfig) || builderConfig.options.tsConfig;
        const parsedTsConfig = readTsConfig(tsConfigPath);
        if (parsedTsConfig.errors.length > 0) {
            const exitCode = ng.exitCodeFromResult(parsedTsConfig.errors);
            if (exitCode !== 0) {
                return Promise.reject(new Error(ng.formatDiagnostics(parsedTsConfig.errors)));
            }
        }
        const entryPoints = context.graph.filter(nodes_1.isEntryPoint);
        for (const entryPointNode of entryPoints) {
            const { entryPoint } = entryPointNode.data;
            log.debug(`Initializing tsconfig for ${entryPoint.moduleId}`);
            const rootDir = Path.dirname(entryPoint.entryFilePath);
            const userOverridingCompilerOptions = nodeLib.compilerOptions || {};
            const overrideOptions = Object.assign({}, userOverridingCompilerOptions, { rootDir, outDir: entryPoint.destinationPath });
            const tsConfig = {
                rootNames: [entryPoint.entryFilePath],
                options: Object.assign(JSON.parse(JSON.stringify(parsedTsConfig.options)), overrideOptions),
                projectReferences: parsedTsConfig.projectReferences,
            };
            build_1.ENTRY_POINT_STORAGE.merge(entryPointNode, {
                nodeLib: { tsConfig }
            });
        }
        return Promise.resolve(context.graph);
    });
}
/**
 * Replace the original compilation step, compile for node and not for angular.
 */
function compilerNgc(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const { epNode } = context;
        const nodeLibCache = build_1.ENTRY_POINT_STORAGE.get(epNode).nodeLib;
        if (context.context().builderConfig.options.watch) {
            if (nodeLibCache.watchProgram) {
                return;
            }
            const formatHost = {
                getCanonicalFileName: path => path,
                getCurrentDirectory: ts.sys.getCurrentDirectory,
                getNewLine: () => ts.sys.newLine
            };
            const entryPoints = context.graph.filter(nodes_1.isEntryPoint);
            // Add paths mappings for dependencies
            const ngParsedTsConfig = tsconfig_1.setDependenciesTsConfigPaths(epNode.data.tsConfig, entryPoints);
            const tsConfig = JSON.parse(JSON.stringify(nodeLibCache.tsConfig));
            tsConfig.options.paths = ngParsedTsConfig.options.paths;
            const host = ts.createWatchCompilerHost(tsConfig.rootNames, tsConfig.options, ts.sys, ts.createEmitAndSemanticDiagnosticsBuilderProgram, (diagnostic) => log.error(ng.formatDiagnostics([diagnostic], formatHost)), (diagnostic) => log.msg(ts.formatDiagnostic(diagnostic, formatHost)), tsConfig.projectReferences);
            const program = ts.createWatchProgram(host);
            nodeLibCache.watchProgram = program;
        }
        else {
            const entryPoints = context.graph.filter(nodes_1.isEntryPoint);
            // Add paths mappings for dependencies
            const ngParsedTsConfig = tsconfig_1.setDependenciesTsConfigPaths(epNode.data.tsConfig, entryPoints);
            const tsConfig = JSON.parse(JSON.stringify(nodeLibCache.tsConfig));
            tsConfig.options.paths = ngParsedTsConfig.options.paths;
            const scriptTarget = tsConfig.options.target;
            const cache = epNode.cache;
            const oldProgram = cache.oldPrograms && cache.oldPrograms[scriptTarget];
            const program = ts.createProgram({
                rootNames: tsConfig.rootNames,
                options: tsConfig.options,
                oldProgram,
            });
            cache.oldPrograms = Object.assign({}, cache.oldPrograms, { [scriptTarget]: program });
            const emitResult = program.emit();
            const allDiagnostics = ts.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);
            log.debug(`ngc program structure is reused: ${oldProgram ? oldProgram.structureIsReused : 'No old program'}`);
            const exitCode = ng.exitCodeFromResult(allDiagnostics);
            if (exitCode !== 0) {
                throw new Error(ng.formatDiagnostics(allDiagnostics));
            }
        }
    });
}
/**
 * Replace the original package.json create logic, write values for a node library.
 */
function writePackage(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const { tsConfig, entryPoint } = context.epNode.data;
        const ngPackage = context.graph.find(node => node.type === 'application/ng-package').data;
        log.info('Writing package metadata');
        const relativeUnixFromDestPath = (filePath) => path_1.ensureUnixPath(Path.relative(entryPoint.destinationPath, filePath));
        // TODO: This will map the entry file to it's emitted output path taking rootDir into account.
        // It might not be fully accurate, consider using the compiler host to create a direct map.
        const distEntryFile = Path.join(entryPoint.destinationPath, Path.relative(tsConfig.options.rootDir, entryPoint.entryFilePath)).replace(/\.ts$/, '.js');
        const distDtsEntryFile = distEntryFile.replace(/\.js$/, '.d.ts');
        yield write_package_transform_1.writePackageJson(entryPoint, ngPackage, {
            main: relativeUnixFromDestPath(distEntryFile),
            typings: relativeUnixFromDestPath(distDtsEntryFile),
        });
        log.success(`Built ${entryPoint.moduleId}`);
    });
}
let NodeLib = class NodeLib {
};
NodeLib = __decorate([
    build_1.Job({
        schema: Path.resolve(__dirname, 'node-lib.json'),
        selector: 'nodeLib',
        internalWatch: true,
        hooks: {
            initTsConfig: {
                after: initTsConfig
            },
            compileNgc: {
                replace: compilerNgc
            },
            writeBundles: {
                replace: []
            },
            writePackage: {
                replace: writePackage
            }
        }
    })
], NodeLib);
exports.NodeLib = NodeLib;
//# sourceMappingURL=node-lib.js.map